/*
UR_RATE.PRG
Specification tests for a threshold bi-variate VECM
on term structure data

written by:

Bruce E. Hansen
Department of Economics
Social Science Building
University of Wisconsin
Madison, WI 53706-1393
bhansen@ssc.wisc.edu
http://www.ssc.wisc.edu/~bhansen/

*/

/*************************************************************/
_trim=.05;		@ trimming percentage for threshold @
boot=5000;		@ number of bootstrap replications @
/*************************************************************/

#include tar.prc;
format 12,6;
output file=ur_rate.out reset;
"**************************************************";"";
"Number of Bootstrap Replications   " boot;
"";"";

load dat[482,62]=zeroyld.dat;
datstore=dat[1:rows(dat),7:62];
rs = seqa(0,1,19)|21|24|30|seqa(36,12,8);

let rates_i[9,2]=
1 2
1 3
1 6
3 6
3 12
3 120
12 24
12 120
24 120;

k=1; do while k<=2;  		@ Lags in VAR beyond EC @

ri=1; do while ri<=rows(rates_i);

short=rates_i[ri,1];
long=rates_i[ri,2];
short_i=maxindc(rs.==short);
long_i=maxindc(rs.==long);
dat=datstore[.,long_i short_i];    

"**************************************************";"";
"Long Rate  (month):     " long;
"Short Rate (month):      " short;
"Number of VAR lags:   " k;
"";

@ Cointegration Tests @
@ Known CI vector @
y = dat[.,1] - dat[.,2];
t=rows(y)-1-k;
@ ADF test @
dy=y[2+k:t+k+1]-y[1+k:t+k];
x=y[1+k:t+k]~ones(t,1);
j=1; do while j<=k;
  x=x~(y[2+k-j:t+k+1-j]-y[1+k-j:t+k-j]);
j=j+1;endo;
rho=dy/x;
e=dy-x*rho;
v=invpd(moment(x,0))*(e'e)/(t-k-2);
adf=rho[1]/sqrt(v[1,1]);
"ADF test 		" adf;

@ Estimated CI vector @
y=dat[.,1];
x=ones(rows(y),1)~dat[.,2];
beta=y/x;
y=y-x*beta;
t=rows(y)-1-k;
@ Engle-Granger test @
dy=y[2+k:t+k+1]-y[1+k:t+k];
x=y[1+k:t+k]~ones(t,1);
j=1; do while j<=k;
  x=x~(y[2+k-j:t+k+1-j]-y[1+k-j:t+k-j]);
j=j+1;endo;
rho=dy/x;
e=dy-x*rho;
v=invpd(moment(x,0))*(e'e)/(t-k-2);
adf=rho[1]/sqrt(v[1,1]);
"Estimated CI vector	" beta';
"Engle-Granger test 	" adf;
"";

@ Nonlinear Tests @
y = dat[.,1] - dat[.,2];
{tests,pvalues} = tar(y,k+1,0,_trim,1-_trim,boot,0,0);
"Test for TAR in EC - Known CI Vector " pvalues[1];
y=dat[.,1]-(ones(rows(y),1)~dat[.,2])*beta;
{tests,pvalues} = tar(y,k+1,0,_trim,1-_trim,boot,0,0);
"Test for TAR in EC - Estimated CI Vector " pvalues[1];
"";"";
ri=ri+1;endo;
k=k+1;endo;

